@@ -2,12 +2,9 @@
 # Copyright (c) The python-semanticversion project
 # This code is distributed under the two-clause BSD License.
 
-from __future__ import unicode_literals
-
 import unittest
-import sys
 
-import semantic_version
+from semantic_version import Version, SimpleSpec, NpmSpec
 
 from .setup_django import django_loaded
 
@@ -16,8 +13,6 @@ if django_loaded:  # pragma: no cover
     from semantic_version import django_fields
     from .django_test_app import models
 
-    import django
-    from django.conf import settings
     from django.core import serializers
     from django.core.management import call_command
     from django.db import connection
@@ -63,32 +58,35 @@ def save_and_refresh(obj):
     obj = obj.__class__.objects.get(id=obj.id)
 
 
-Version = semantic_version.Version
-Spec = semantic_version.Spec
-
-
 @unittest.skipIf(not django_loaded, "Django not installed")
 class DjangoFieldTestCase(unittest.TestCase):
     def test_version(self):
-        obj = models.VersionModel(version=Version('0.1.1'), spec=Spec('==0.1.1,!=0.1.1-alpha'))
+        obj = models.VersionModel(
+            version=Version('0.1.1'),
+            spec=SimpleSpec('==0.1.1,!=0.1.1-alpha'),
+            npm_spec=NpmSpec('1.2 - 2.3'),
+        )
 
         self.assertEqual(Version('0.1.1'), obj.version)
-        self.assertEqual(Spec('==0.1.1,!=0.1.1-alpha'), obj.spec)
+        self.assertEqual(SimpleSpec('==0.1.1,!=0.1.1-alpha'), obj.spec)
+        self.assertEqual(NpmSpec('1.2 - 2.3'), obj.npm_spec)
 
-        alt_obj = models.VersionModel(version=obj.version, spec=obj.spec)
+        alt_obj = models.VersionModel(version=obj.version, spec=obj.spec, npm_spec=obj.npm_spec)
 
         self.assertEqual(Version('0.1.1'), alt_obj.version)
-        self.assertEqual(Spec('==0.1.1,!=0.1.1-alpha'), alt_obj.spec)
+        self.assertEqual(SimpleSpec('==0.1.1,!=0.1.1-alpha'), alt_obj.spec)
         self.assertEqual(obj.spec, alt_obj.spec)
+        self.assertEqual(obj.npm_spec, alt_obj.npm_spec)
         self.assertEqual(obj.version, alt_obj.version)
 
     def test_version_clean(self):
         """Calling .full_clean() should convert str to Version/Spec objects."""
-        obj = models.VersionModel(version='0.1.1', spec='==0.1.1,!=0.1.1-alpha')
+        obj = models.VersionModel(version='0.1.1', spec='==0.1.1,!=0.1.1-alpha', npm_spec='1.x')
         obj.full_clean()
 
         self.assertEqual(Version('0.1.1'), obj.version)
-        self.assertEqual(Spec('==0.1.1,!=0.1.1-alpha'), obj.spec)
+        self.assertEqual(SimpleSpec('==0.1.1,!=0.1.1-alpha'), obj.spec)
+        self.assertEqual(NpmSpec('1.x'), obj.npm_spec)
 
     def test_version_save(self):
         """Test saving object with a VersionField."""
@@ -98,13 +96,13 @@ class DjangoFieldTestCase(unittest.TestCase):
         self.assertIsNone(obj.optional)
         save_and_refresh(obj)
         self.assertIsNotNone(obj.id)
-        self.assertIsNone(obj.optional_spec)
+        self.assertIsNone(obj.optional)
 
         # now set to something that is not null
-        spec = Spec('==0,!=0.2')
-        obj.optional_spec = spec
+        version = Version('1.2.3')
+        obj.optional = version
         save_and_refresh(obj)
-        self.assertEqual(obj.optional_spec, spec)
+        self.assertEqual(obj.optional, version)
 
     def test_spec_save(self):
         """Test saving object with a SpecField."""
@@ -117,7 +115,7 @@ class DjangoFieldTestCase(unittest.TestCase):
         self.assertIsNone(obj.optional_spec)
 
         # now set to something that is not null
-        spec = Spec('==0,!=0.2')
+        spec = SimpleSpec('==0,!=0.2')
         obj.optional_spec = spec
         save_and_refresh(obj)
         self.assertEqual(obj.optional_spec, spec)
@@ -126,7 +124,7 @@ class DjangoFieldTestCase(unittest.TestCase):
         obj = models.VersionModel(version='0.1.1', spec='==0,!=0.2')
         obj.full_clean()
         self.assertEqual(Version('0.1.1'), obj.version)
-        self.assertEqual(Spec('==0,!=0.2'), obj.spec)
+        self.assertEqual(SimpleSpec('==0,!=0.2'), obj.spec)
 
     def test_coerce_clean(self):
         obj = models.CoerceVersionModel(version='0.1.1a+2', partial='23')
@@ -139,7 +137,6 @@ class DjangoFieldTestCase(unittest.TestCase):
         self.assertEqual(Version('23.0.0'), obj2.version)
         self.assertEqual(Version('0.1.2+3.4.5-6', partial=True), obj2.partial)
 
-    @unittest.skipIf(django.VERSION[:2] < (1, 8), "Django<1.8 casts values on setattr")
     def test_invalid_input(self):
         v = models.VersionModel(version='0.1.1', spec='blah')
         self.assertRaises(ValueError, v.full_clean)
@@ -147,11 +144,6 @@ class DjangoFieldTestCase(unittest.TestCase):
         v2 = models.VersionModel(version='0.1', spec='==0.1.1,!=0.1.1-alpha')
         self.assertRaises(ValueError, v2.full_clean)
 
-    @unittest.skipUnless(django.VERSION[:2] < (1, 8), "Django>=1.8 doesn't mangle setattr")
-    def test_invalid_input_full_clean(self):
-        self.assertRaises(ValueError, models.VersionModel, version='0.1.1', spec='blah')
-        self.assertRaises(ValueError, models.VersionModel, version='0.1', spec='==0.1.1,!=0.1.1-alpha')
-
     def test_partial(self):
         obj = models.PartialVersionModel(partial=Version('0.1.0'))
 
@@ -175,17 +167,26 @@ class DjangoFieldTestCase(unittest.TestCase):
         obj.full_clean()
 
     def test_serialization(self):
-        o1 = models.VersionModel(version=Version('0.1.1'), spec=Spec('==0.1.1,!=0.1.1-alpha'))
-        o2 = models.VersionModel(version=Version('0.4.3-rc3+build3'),
-            spec=Spec('<=0.1.1-rc2,!=0.1.1-rc1'))
+        o1 = models.VersionModel(
+            version=Version('0.1.1'),
+            spec=SimpleSpec('==0.1.1,!=0.1.1-alpha'),
+            npm_spec=NpmSpec('1.2 - 2.3'),
+        )
+        o2 = models.VersionModel(
+            version=Version('0.4.3-rc3+build3'),
+            spec=SimpleSpec('<=0.1.1-rc2,!=0.1.1-rc1'),
+            npm_spec=NpmSpec('1.2 - 2.3'),
+        )
 
         data = serializers.serialize('json', [o1, o2])
 
         obj1, obj2 = serializers.deserialize('json', data)
         self.assertEqual(o1.version, obj1.object.version)
         self.assertEqual(o1.spec, obj1.object.spec)
+        self.assertEqual(o1.npm_spec, obj1.object.npm_spec)
         self.assertEqual(o2.version, obj2.object.version)
         self.assertEqual(o2.spec, obj2.object.spec)
+        self.assertEqual(o2.npm_spec, obj2.object.npm_spec)
 
     def test_serialization_partial(self):
         o1 = models.PartialVersionModel(
@@ -196,7 +197,7 @@ class DjangoFieldTestCase(unittest.TestCase):
         o2 = models.PartialVersionModel(
             partial=Version('0.4.3-rc3+build3', partial=True),
             optional='',
-            optional_spec=Spec('==0.1.1,!=0.1.1-alpha'),
+            optional_spec=SimpleSpec('==0.1.1,!=0.1.1-alpha'),
         )
 
         data = serializers.serialize('json', [o1, o2])
@@ -220,19 +221,16 @@ class FieldMigrationTests(DjangoTestCase):
             'partial': True,
             'max_length': 200,
         }
-        if hasattr(sys, 'pypy_version_info'):
-            # Django under Pypy adds this extra key.
-            expected['help_text'] = u''
-
         self.assertEqual(field.deconstruct()[3], expected)
 
     def test_spec_field(self):
         field = django_fields.SpecField()
         expected = {'max_length': 200}
-        if hasattr(sys, 'pypy_version_info'):
-            # Django under Pypy adds this extra key.
-            expected['help_text'] = u''
+        self.assertEqual(field.deconstruct()[3], expected)
 
+    def test_nondefault_spec_field(self):
+        field = django_fields.SpecField(syntax='npm')
+        expected = {'max_length': 200, 'syntax': 'npm'}
         self.assertEqual(field.deconstruct()[3], expected)
 
 
@@ -244,8 +242,7 @@ class FullMigrateTests(TransactionTestCase):
         call_command('migrate', verbosity=0)
         with connection.cursor() as cursor:
             table_list = connection.introspection.get_table_list(cursor)
-            if django.VERSION[:2] >= (1, 8):
-                table_list = [t.name for t in connection.introspection.get_table_list(cursor)]
+            table_list = [t.name for t in connection.introspection.get_table_list(cursor)]
             self.assertIn('django_test_app_versionmodel', table_list)
 
 
@@ -253,8 +250,8 @@ class FullMigrateTests(TransactionTestCase):
 class DbInteractingTestCase(DjangoTestCase):
 
     def test_db_interaction(self):
-        o1 = models.VersionModel(version=Version('0.1.1'), spec=Spec('<0.2.4-rc42'))
-        o2 = models.VersionModel(version=Version('0.4.3-rc3+build3'), spec=Spec('==0.4.3'))
+        o1 = models.VersionModel(version=Version('0.1.1'), spec=SimpleSpec('<0.2.4-rc42'))
+        o2 = models.VersionModel(version=Version('0.4.3-rc3+build3'), spec=SimpleSpec('==0.4.3'))
 
         o1.save()
         o2.save()
